//      

                                                                         

class Debouncer                               {
  /**
   * A map of commands in-flight to debounce.
   *
   * @var {InflightStore}
   */
                          

  /**
   * The store that resolves the commands.
   *
   * @var {Object}
   */
                  

  /**
   * @inheritDoc
   */
  constructor(inflight               , backend        )       {
    this.inflight = inflight;
    this.backend = backend;
  }

  /**
   * @inheritDoc
   */
  debounce(command        , ...args            )             {
    const key = [command, args];
    const debounced              = this.inflight.get(key);
    if (debounced) {
      return debounced;
    }
    const promise = this.backend[command](...args).then((result) => {
      // Remove from in-flight when it resolves and return the results.
      this.inflight.delete(key);
      return result;
    });
    // Add the promise to the in-flight map.
    this.inflight.set(key, promise);
    return promise;
  }
}

module.exports = Debouncer;
